@@ -1,5 +1,5 @@
 # Pod::Text::Overstrike -- Convert POD data to formatted overstrike text
-# $Id: Overstrike.pm,v 1.4 2001/11/15 08:04:18 eagle Exp $
+# $Id: Overstrike.pm,v 1.5 2001/11/23 09:57:15 eagle Exp $
 #
 # Created by Joe Smith <Joe.Smith@inwap.com> 30-Nov-2000
 #   (based on Pod::Text::Color by Russ Allbery <rra@stanford.edu>)
@@ -36,7 +36,7 @@ use vars qw(@ISA $VERSION);
 # Don't use the CVS revision as the version, since this module is also in Perl
 # core and too many things could munge CVS magic revision strings.  This
 # number should ideally be the same as the CVS revision in podlators, however.
-$VERSION = 1.04;
+$VERSION = 1.05;
 
 
 ##############################################################################
@@ -45,41 +45,53 @@ $VERSION = 1.04;
 
 # Make level one headings bold, overridding any existing formatting.
 sub cmd_head1 {
-    my $self = shift;
-    local $_ = shift;
-    s/\s+$//;
-    s/(.)\cH\1//g;
-    s/_\cH//g;
-    s/(.)/$1\b$1/g;
-    $self->SUPER::cmd_head1 ($_);
+    my ($self, $text, $line) = @_;
+    $text =~ s/\s+$//;
+    $text = $self->strip_format ($self->interpolate ($text, $line));
+    $text =~ s/(.)/$1\b$1/g;
+    $self->SUPER::cmd_head1 ($text);
 }
 
 # Make level two headings bold, overriding any existing formatting.
 sub cmd_head2 {
-    my $self = shift;
-    local $_ = shift;
-    s/\s+$//;
-    s/(.)\cH\1//g;
-    s/_\cH//g;
-    s/(.)/$1\b$1/g;
-    $self->SUPER::cmd_head2 ($_);
+    my ($self, $text, $line) = @_;
+    $text =~ s/\s+$//;
+    $text = $self->strip_format ($self->interpolate ($text, $line));
+    $text =~ s/(.)/$1\b$1/g;
+    $self->SUPER::cmd_head2 ($text);
 }
 
 # Make level three headings underscored, overriding any existing formatting.
 sub cmd_head3 {
-    my $self = shift;
-    local $_ = shift;
-    s/\s+$//;
-    s/(.)\cH\1//g;
-    s/_\cH//g;
-    s/(.)/_\b$1/g;
-    $self->SUPER::cmd_head3 ($_);
+    my ($self, $text, $line) = @_;
+    $text =~ s/\s+$//;
+    $text = $self->strip_format ($self->interpolate ($text, $line));
+    $text =~ s/(.)/_\b$1/g;
+    $self->SUPER::cmd_head3 ($text);
+}
+
+# Level four headings look like level three headings.
+sub cmd_head4 {
+    my ($self, $text, $line) = @_;
+    $text =~ s/\s+$//;
+    $text = $self->strip_format ($self->interpolate ($text, $line));
+    $text =~ s/(.)/_\b$1/g;
+    $self->SUPER::cmd_head4 ($text);
+}
+
+# The common code for handling all headers.  We have to override to avoid
+# interpolating twice and because we don't want to honor alt.
+sub heading {
+    my ($self, $text, $line, $indent, $marker) = @_;
+    $self->item ("\n\n") if defined $$self{ITEM};
+    $text .= "\n" if $$self{loose};
+    $self->output (' ' x $indent . $text . "\n");
 }
 
 # Fix the various interior sequences.
-sub seq_b { local $_ = $_[1]; s/(.)\cH\1//g; s/_\cH//g; s/(.)/$1\b$1/g; $_ }
-sub seq_f { local $_ = $_[1]; s/(.)\cH\1//g; s/_\cH//g; s/(.)/_\b$1/g; $_ }
-sub seq_i { local $_ = $_[1]; s/(.)\cH\1//g; s/_\cH//g; s/(.)/_\b$1/g; $_ }
+sub seq_b { local $_ = strip_format (@_); s/(.)/$1\b$1/g; $_ }
+sub seq_f { local $_ = strip_format (@_); s/(.)/_\b$1/g; $_ }
+sub seq_i { local $_ = strip_format (@_); s/(.)/_\b$1/g; $_ }
 
 # Output any included code in bold.
 sub output_code {
@@ -109,6 +121,19 @@ sub wrap {
     $output;
 }
 
+##############################################################################
+# Utility functions
+##############################################################################
+
+# Strip all of the formatting from a provided string, returning the stripped
+# version.
+sub strip_format {
+    my ($self, $text) = @_;
+    $text =~ s/(.)\cH\1/$1/g;
+    $text =~ s/_\cH//g;
+    return $text;
+}
+
 ##############################################################################
 # Module return value and documentation
 ##############################################################################
